local AceGUI = LibStub("AceGUI-3.0")

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent

-------------
-- Widgets --
-------------
--[[
	Widgets must provide the following functions
		Acquire() - Called when the object is aquired, should set everything to a default hidden state
		Release() - Called when the object is Released, should remove any anchors and hide the Widget
		
	And the following members
		frame - the frame or derivitive object that will be treated as the widget for size and anchoring purposes
		type - the type of the object, same as the name given to :RegisterWidget()
		
	Widgets contain a table called userdata, this is a safe place to store data associated with the wigdet
	It will be cleared automatically when a widget is released
	Placing values directly into a widget object should be avoided
	
	If the Widget can act as a container for other Widgets the following
		content - frame or derivitive that children will be anchored to
		
	The Widget can supply the following Optional Members


]]

--------------------------
-- Inline Group		 --
--------------------------
--[[
	This is a simple grouping container, no selection
	It will resize automatically to the height of the controls added to it
]]

do
	local Type = "UIF-InlineGroup"
	local Version = 3
	
	local function SetTitle(self,title)
		self.titletext:SetText(title)
	end
	
	local function SetFont(self, font, size, flags)
		self.titletext:SetFont(font, size, flags)
	end
	
	local function SetFontColor(self, color)
		self.titletext:SetTextColor(unpack(color))
	end
	
	local function SetBGTexture(self, texture)
		PaneBackdrop.bgFile = texture
		self.border:SetBackdrop(PaneBackdrop)
	end
	
	local function SetBorderTexture(self, texture)
		PaneBackdrop.edgeFile = texture
		self.border:SetBackdrop(PaneBackdrop)
	end
	
	local function SetBGColor(self, color)
		self.border:SetBackdropColor(unpack(color))		
	end
	
	local function SetBorderColor(self, color)
		self.border:SetBackdropBorderColor(unpack(color))
	end

	
	local function Constructor()
		local self = AceGUI:Create("InlineGroup")
		self.type = Type
		
		self.SetFont = SetFont
		self.SetFontColor = SetFontColor
		self.SetBGTexture = SetBGTexture
		self.SetBorderTexture = SetBorderTexture
		self.SetBGColor = SetBGColor
		self.SetBorderColor = SetBorderColor
		
		return self
	end
	
	AceGUI:RegisterWidgetType(Type,Constructor,Version)
end
