local mod = RW:Boss("The Beasts of Northrend", "Crusaders Coliseum")
local L

mod.BossIcon = [[Interface\Icons\ability_hunter_pet_worm]]
mod:SetPackIcon([[Interface\ICONS\achievement_reputation_argentchampion]])
mod.SupressGlobalHealthFrames = true

------------------------------------
-- Variables

--Gormok
local tStompCD, tImpaleCD, tImpaleDur
local mImpale, mSnoboll
local wFireBomb, wSnoboll
--Worms
local tBurrow, tPoolCD
local mBile, mToxin, mPool
local wBile, wToxin, wPool
local iToxin, iBurn
--Icehowl
local tCharge, tDazeDur, tCrashCD
local mCharge, mFrot
local wCharge
local iCharge
--General
local mPhase
local tNext
local tBerserk

-----------------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L.YellPull = "Hailing from the deepest, darkest caverns of the Storm Peaks, Gormok the Impaler! Battle on, heroes!"
		L.YellP2 = "Steel yourselves, heroes, for the twin terrors, Acidmaw and Dreadscale, enter the arena!"
		L.YellP3 = "The air itself freezes with the introduction of our next combatant, Icehowl! Kill or be killed, champions!"
	
		L.burrow = "Burrow cooldown"
		L.burrow_msg = "Next Burrow"
		
		L.health_frames = "Health Frames"
		
		L.next_msg = "%s incomming!"
		L.next = "Next boss timer"
		
		L.gormok 		= "Gormok"
		L.jormungars 	= "Jormungars"
		L.icehowl 		= "Icehowl"
		
		L.charge_target = "^%%s glares at (%S+) and lets out"
	end
	L = mod:GetLocale()
	mod.L = L
	
	mod:SetTrigger(34796, 35144, 34799, 34797, L.YellPull)
	mod:SetWin(34797)	
	mod:SetDelay(20)
	
	self:AddToCategory(L.gormok, 67647, 66331, 66406, 67472)
	self:AddToCategory(L.jormungars, 66883, L.burrow, 66869, 67618)
	self:AddToCategory(L.icehowl, 52311, 66758, 67662, 67657)
	
	-- Gormok
	tStompCD, tImpaleCD, tImpaleDur	= self:CDTimer(21, 67647), 			self:CDTimer(10, 66331), 		self:DurTargetStackTimer(40, 66331)
	mImpale, mSnoboll 				= self:TargetStackMessage(66331),	self:TargetMessage(66406)
	wSnoboll, wFireBomb				= self:Warning(66406), 				self:RunawayWarning(67472)
	
	-- Worms
	tPoolCD, tBurrow 				= self:CDTimer(12, 66883, false), 	self:CustomTimer(45, L.burrow_msg, L.burrow.." timer", [[Interface\Icons\Ability_Smash]], L.burrow)
	mBile, mToxin, mPool			= self:TargetMessage(66869), 		self:TargetMessage(67618), 		self:Message(66883)
	wBile, wToxin, wPool			= self:RunawayWarning(66869), 		self:RunawayWarning(67618),		self:RunawayWarning(66881)
	iToxin							= self:Icon("STAR", 67618, nil, nil, "CIRCLE", "DIAMOND", "TRIANGLE")
	iBurn							= self:Icon("MOON", 66869, nil, nil, "SQUARE", "CROSS", "SKULL")
	
	-- Icehowl
	tCharge, tDazeDur, tCrashCD		= self:CastTimer(52311),			self:DurTimer(15, 66758),		self:CDTimer(40, 67662)
	mCharge, mFrot					= self:TargetMessage(52311),		self:Message(67657)
	wCharge							= self:RunawayWarning(52311)
	iCharge							= self:Icon("SKULL", 52311, nil, 10)
	
	-- General
	mPhase	= self:PhaseMessage()
	tNext	= self:CustomTimer(180, L.next_msg, L.next, [[Interface\Icons\Achievement_BG_killingblow_most]], L.next)
	tBerserk = self:BerserkTimer(220)
end

-----------------------------------------
-- locals

local impaleDur
local phase
local snobolled = GetSpellInfo(66406)
local bollSpamm, bileTargets, toxinTargets = {}, {}, {}
local burnSpamm, poolSpam
local impaleDur = {
	30, 40, 30, 45
}

-----------------------------------------
-- Start

function mod:OnEngage()
	self:Track("Impale", 67477, 66331, 67478, 67479)
	self:Track("Stomp", 67647, 66330, 67649, 67648)
	self:Track("FireBomb", 67472, 66317, 66320)
	
	self:Track("Bile", 66869, 67621, 67622, 67623, 66870)
	self:Track("Toxin", 67618, 67619, 67620, 66823)
	self:Track("Pool", 66883, 67641, 67642, 67643)
	
	self:Track("Frot", 66759, 67657, 67658, 67659)
	self:Track("Daze", 66758)
	
	self:YellTrack("Phase2", L.YellP2)
	self:YellTrack("Phase3", L.YellP3)
	
	self:RegisterEvent("UNIT_AURA")
	
	wipe(bollSpamm)
	wipe(bileTargets)
	wipe(toxinTargets)
	burnSpamm = 0
	poolSpam = 0
	phase = 1	
	
	tNext:Start(22, L.gormok)
	if self:IsHeroic() then
		tNext:Start(180, L.jormungars)
	end
end

----------
-- Gormokk
function mod:UNIT_AURA(event, uID)
	if phase ~= 1 then return end
	local bolled = UnitDebuff(uID, snobolled)
	local name = UnitName(uID)
	if bollSpamm[name] and not bolled then
		bollSpamm[name] = nil
	elseif bolled and not bollSpamm[name] then
		bollSpamm[name] = true
		mSnoboll:Show(name)
		if name == self.pName then
			wSnoboll:Show()
		end
	end
end

function mod:Impale(event, args)
	if event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_APPLIED_DOSE" then		
		tImpaleDur:Start(impaleDur[GetInstanceDifficulty()], args.dName, args.amount or 1)
		mImpale:Show(args.dName, args.amount or 1)
		tImpaleCD:Start()
	end
end

function mod:Stomp(event, args)
	if event == "SPELL_CAST_START" then
		tStompCD:Start()
	end
end

function mod:FireBomb(event, args)
	if event == "SPELL_DAMAGE" then
		if args:IsPlayer() then
			local t = GetTime()
			if t > (burnSpamm + 4) then
				wFireBomb:Show()
				burnSpamm = t
			end
		end
	end
end

---------
-- Worms
function mod:ShowBile()
	mBile:Show(table.concat(bileTargets, "<, >"))
	wipe(bileTargets)
end

function mod:Bile(event, args)
	if event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			wBile:Show()
		end
		iBurn:Show(args.dName)
		self:Unschedule("Bile")
		bileTargets[#bileTargets+1] = args.dName
		self:Schedule("Bile", 0.2, "ShowBile")
	elseif event == "SPELL_AURA_REMOVED" then
		iBurn:Hide(args.dName)
	end
end

function mod:ShowToxin()
	mToxin:Show(table.concat(toxinTargets, "<, >"))
	wipe(toxinTargets)
end

function mod:Toxin(event, args)
	if event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			wToxin:Show()
		end
		iToxin:Show(args.dName)
		self:Unschedule("Toxin")
		toxinTargets[#toxinTargets+1] = args.dName
		self:Schedule("Toxin", 0.2, "ShowToxin")
	elseif event == "SPELL_AURA_REMOVED" then
		iToxin:Hide(args.dName)
	end
end

function mod:Pool(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		tPoolCD:Start()
		mPool:Show()
		
	elseif event == "SPELL_DAMAGE" then		
		if args:IsPlayer() then	
			local t = GetTime()
			if t > (poolSpam + 4) then
				wPool:Show()
				poolSpam = t
			end
		end
	end
end

----------
-- Icehowl
function mod:Frot(event, args)
	if event == "SPELL_AURA_APPLIED" then
		mFrot:Show()
	end
end

function mod:Daze(event, args)	
	if event == "SPELL_AURA_APPLIED" then
		tDazeDur:Start()
	end
end

function mod:CHAT_MSG_RAID_BOSS_EMOTE(event, msg, name)
	local target = msg:match(L.charge_target)
	
	if target then
		if target == self.pName then
			wCharge:Show()
		elseif self.Party.group[target] then
			local uID = self.Party.group[target].id
			if uID and CheckInteractDistance(uID, 2) then
				wCharge:Show()
			end
		end
		mCharge:Show(target)
		iCharge:Show(target)
	end
	tCharge:Start()
	tCrashCD:Schedule(8)
end

---------
-- Phases

function mod:BurrowFunc()
	tBurrow:ScheduleRepeating(54, true, 53)
end

function mod:Phase2(event, msg, name)
	self:UnregisterEvent("UNIT_AURA")
	mPhase:Show(2)
	phase = 2
	tBurrow:Schedule(15)
	self:Schedule("BurrowTimer", 60, "BurrowFunc")
	if self:IsHeroic() then
		tNext:Stop()
		tNext:Start(200, L.icehowl)
	end
	self:SendMessage("RW_AddHealthFrame", 34799)
	self:SendMessage("RW_AddHealthFrame", 35144)
	
end

function mod:Phase3(event, msg, name)
	mPhase:Show(3)
	phase = 3
	self:Unschedule("BurrowTimer")
	tBurrow:Unschedule()
	tBurrow:Stop()
	tNext:Stop()
	self:SendMessage("RW_RemoveAllHealthFrames")
	if self:IsHeroic() then
		tBerserk:Start()
	end
	tCrashCD:Start()
end